@@ -1,12 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- *  Copyright (C) 2006  James Livingston  <jrl@ids.org.au>
+ *  Copyright (C) 2006  James Livingston  <doclivingston@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -34,16 +42,55 @@ enum {
 	TICK,
 	EVENT,
 	PLAYING_STREAM,
+	VOLUME_CHANGED,
+	IMAGE,
 	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+/**
+ * SECTION:rb-player
+ * @short_description: playback backend interface
+ * @include: rb-player.h
+ *
+ * This is the interface implemented by the rhythmbox playback backends.
+ * It allows the caller to control playback (open, play, pause, close), 
+ * seek (set_time), control volume (get_volume, set_volume, set_replaygain)
+ * and receive playback state information (get_time, various signals).
+ *
+ * The playback interface allows for multiple streams to be playing (or at
+ * least open) concurrently. The caller associates some data with each stream
+ * it opens (#rb_player_open), which is included in the paramters with each
+ * signal emitted. The caller should not assume that the new stream is playing
+ * immediately upon returning from #rb_player_play. Instead, it should use
+ * the 'playing-stream' signal to determine that.
+ *
+ * The player implementation should emit signals for metadata extracted from the
+ * stream using the 'info' signal
+ *
+ * While playing, the player implementation should emit 'tick' signals frequently
+ * enough to update an elapsed/remaining time display consistently.  The duration
+ * value included in tick signal emissions is used to prepare the next stream before
+ * the current stream reaches EOS, so it should be updated for each emission to account
+ * for variable bitrate streams that produce inaccurate duration estimates early on.
+ *
+ * When playing a stream from the network, the player can report buffering status
+ * using the 'buffering' signal.  The value included in the signal indicates the
+ * percentage of the buffer that has been filled.
+ *
+ * The 'event' signal can be used to communicate events from the player to the application.
+ * For GStreamer-based player implementations, events are triggered by elements in the
+ * pipeline posting application messages.  The name of the message becomes the name of the
+ * event.
+ */
+
 static void
 rb_player_interface_init (RBPlayerIface *iface)
 {
 	/**
-	 * RBPlayer::eos
+	 * RBPlayer::eos:
+	 * @player: the #RBPlayer
 	 * @stream_data: the data associated with the stream that finished
 	 *
 	 * The 'eos' signal is emitted when a stream finishes.
@@ -59,7 +106,8 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      1, G_TYPE_POINTER);
 
 	/**
-	 * RBPlayer::info
+	 * RBPlayer::info:
+	 * @player: the #RBPlayer
 	 * @stream_data: the data associated with the stream
 	 * @field: the #RBMetaDataField corresponding to the stream info
 	 * @value: the value of the stream info field
@@ -78,7 +126,8 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      3, G_TYPE_POINTER, G_TYPE_INT, G_TYPE_VALUE);
 
 	/**
-	 * RBPlayer::error
+	 * RBPlayer::error:
+	 * @player: the #RBPlayer
 	 * @stream_data: the data associated with the stream
 	 * @error: description of the error
 	 *
@@ -97,10 +146,12 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      G_TYPE_POINTER, G_TYPE_POINTER);
 
 	/**
-	 * RBPlayer::tick
+	 * RBPlayer::tick:
+	 * @player: the #RBPlayer
 	 * @stream_data: the data associated with the stream
-	 * @elapsed: playback position in the stream
+	 * @elapsed: playback position in the stream (in nanoseconds)
 	 * @duration: current estimate of the duration of the stream
+	 *  (in nanoseconds)
 	 *
 	 * The 'tick' signal is emitted repeatedly while the stream is
 	 * playing. Signal handlers can use this to update UI and to
@@ -112,13 +163,14 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBPlayerIface, tick),
 			      NULL, NULL,
-			      rb_marshal_VOID__POINTER_LONG_LONG,
+			      rb_marshal_VOID__POINTER_INT64_INT64,
 			      G_TYPE_NONE,
 			      3,
-			      G_TYPE_POINTER, G_TYPE_LONG, G_TYPE_LONG);
+			      G_TYPE_POINTER, G_TYPE_INT64, G_TYPE_INT64);
 
 	/**
-	 * RBPlayer::buffering
+	 * RBPlayer::buffering:
+	 * @player: the #RBPlayer
 	 * @stream_data: the data associated with the buffering stream
 	 * @progress: buffering percentage
 	 *
@@ -138,7 +190,8 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      G_TYPE_POINTER, G_TYPE_UINT);
 
 	/**
-	 * RBPlayer::event
+	 * RBPlayer::event:
+	 * @player: the #RBPlayer
 	 * @stream_data: data associated with the stream
 	 * @data: event data
 	 *
@@ -161,12 +214,13 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      G_TYPE_POINTER, G_TYPE_POINTER);
 
 	/**
-	 * RBPlayer::playing-stream
+	 * RBPlayer::playing-stream:
+	 * @player: the #RBPlayer
 	 * @stream_data: data associated with the stream
 	 *
 	 * The 'playing-stream' signal is emitted when the main playing stream
 	 * changes. It should be used to update the UI to show the new
-	 * stream. It can either be emitted before or after rb_player_play returns,
+	 * stream. It can either be emitted before or after #rb_player_play returns,
 	 * depending on the player backend.
 	 */
 	signals[PLAYING_STREAM] =
@@ -179,6 +233,44 @@ rb_player_interface_init (RBPlayerIface *iface)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_POINTER);
+	/**
+	 * RBPlayer::volume-changed:
+	 * @player: the #RBPlayer
+	 * @volume: the new volume level
+	 *
+	 * The 'volume-changed' signal is emitted when the output stream volume is
+	 * changed externally.
+	 */
+	signals[VOLUME_CHANGED] =
+		g_signal_new ("volume-changed",
+			      G_TYPE_FROM_INTERFACE (iface),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerIface, volume_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__FLOAT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_FLOAT);
+	
+	/**
+	 * RBPlayer::image:
+	 * @player: the #RBPlayer
+	 * @stream_data: data associated with the stream
+	 * @image: the image extracted from the stream
+	 *
+	 * The 'image' signal is emitted to provide access to images extracted
+	 * from the stream.
+	 */
+	signals[IMAGE] =
+		g_signal_new ("image",
+			      G_TYPE_FROM_INTERFACE (iface),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerIface, image),
+			      NULL, NULL,
+			      rb_marshal_VOID__POINTER_OBJECT,
+			      G_TYPE_NONE,
+			      2,
+			      G_TYPE_POINTER, GDK_TYPE_PIXBUF);
 }
 
 GType
@@ -217,9 +309,9 @@ rb_player_get_type (void)
  * implementation, this may stop any existing stream being
  * played.  The stream preparation process may continue
  * asynchronously, in which case errors may be reported from
- * rb_player_play or using the 'error' signal.
+ * #rb_player_play or using the 'error' signal.
  *
- * @returns TRUE if the stream preparation was not unsuccessful
+ * Return value: TRUE if the stream preparation was not unsuccessful
  */
 gboolean
 rb_player_open (RBPlayer *player, const char *uri, gpointer stream_data, GDestroyNotify stream_data_destroy, GError **error)
@@ -232,7 +324,8 @@ rb_player_open (RBPlayer *player, const char *uri, gpointer stream_data, GDestro
 /**
  * rb_player_opened:
  * @player: 	a #RBPlayer
- * @returns TRUE if a stream is prepared for playback
+ *
+ * Return value: TRUE if a stream is prepared for playback
  */
 gboolean
 rb_player_opened (RBPlayer *player)
@@ -255,7 +348,7 @@ rb_player_opened (RBPlayer *player)
  * If no streams remain open after this call, the audio device will
  * be released.
  *
- * @returns TRUE if a stream was found and closed
+ * Return value: TRUE if a stream was found and closed
  */
 gboolean
 rb_player_close (RBPlayer *player, const char *uri, GError **error)
@@ -268,34 +361,37 @@ rb_player_close (RBPlayer *player, const char *uri, GError **error)
 /**
  * rb_player_play:
  * @player:	a #RBPlayer
- * @crossfade:	requested crossfade duration
+ * @play_type:  requested playback start type
+ * @crossfade:	requested crossfade duration (nanoseconds)
  * @error:	returns error information
  *
  * Starts playback of the most recently opened stream.
- * If @crossfade is greater than zero, the player may attempt
- * to crossfade the new stream with any existing streams.
+ * if @play_type is #RB_PLAYER_PLAY_CROSSFADE, the player
+ * may attempt to crossfade the new stream with any existing
+ * streams.  If it does this, the it will use @crossfade as the
+ * duration of the fade.
  *
- * If @crossfade is zero, the player may attempt to start the
- * stream immediately after the current playing stream reaches
- * EOS.  This may or may not result in the phenomemon known
- * as 'gapless playback'.
+ * If @play_type is #RB_PLAYER_PLAY_AFTER_EOS, the player may
+ * attempt to start the stream immediately after the current
+ * playing stream reaches EOS.  This may or may not result in
+ * the phenomemon known as 'gapless playback'.
  *
- * If @crossfade is less than zero, the player will stop any
+ * If @play_type is #RB_PLAYER_PLAY_REPLACE, the player will stop any
  * existing stream before starting the new stream. It may do
- * this anyway, regardless of the value of @crossfade.
+ * this anyway, regardless of the value of @play_type.
  *
  * The 'playing-stream' signal will be emitted when the new stream
  * is actually playing. This may be before or after control returns
  * to the caller.
  *
- * @returns: TRUE if playback started successfully
+ * Return value: %TRUE if playback started successfully
  */
 gboolean
-rb_player_play (RBPlayer *player, gint crossfade, GError **error)
+rb_player_play (RBPlayer *player, RBPlayerPlayType play_type, gint64 crossfade, GError **error)
 {
 	RBPlayerIface *iface = RB_PLAYER_GET_IFACE (player);
 
-	return iface->play (player, crossfade, error);
+	return iface->play (player, play_type, crossfade, error);
 }
 
 /**
@@ -316,8 +412,9 @@ rb_player_pause (RBPlayer *player)
 
 /**
  * rb_player_playing:
- * @player:	a #RBPlayer
- * @returns:	TRUE if a stream is currently being played (not paused
+ * @player:	a #RBPlayer.
+ *
+ * Return value: TRUE if a stream is currently being played (not paused
  *  or being faded out).
  */
 gboolean
@@ -348,7 +445,8 @@ rb_player_set_volume (RBPlayer *player, float volume)
 /**
  * rb_player_get_volume:
  * @player:	a #RBPlayer
- * @returns:	current output volume level
+ *
+ * Return value: current output volume level
  */
 float
 rb_player_get_volume (RBPlayer *player)
@@ -383,7 +481,8 @@ rb_player_set_replaygain (RBPlayer *player,
 /**
  * rb_player_seekable:
  * @player:	a #RBPlayer
- * @returns:	TRUE if the current stream is seekable
+ *
+ * Return value: TRUE if the current stream is seekable
  */
 gboolean
 rb_player_seekable (RBPlayer *player)
@@ -396,26 +495,28 @@ rb_player_seekable (RBPlayer *player)
 /**
  * rb_player_set_time:
  * @player:	a #RBPlayer
- * @time:	seek target position in seconds
+ * @newtime:	seek target position in seconds
  *
  * Attempts to seek in the current stream.  The player
  * may ignore this if the stream is not seekable.
  * The seek may take place asynchronously.
  */
 void
-rb_player_set_time (RBPlayer *player, long time)
+rb_player_set_time (RBPlayer *player, gint64 newtime)
 {
 	RBPlayerIface *iface = RB_PLAYER_GET_IFACE (player);
 
-	iface->set_time (player, time);
+	iface->set_time (player, newtime);
 }
 
 /**
  * rb_player_get_time:
  * @player:	a #RBPlayer
- * @returns:	the current playback position in the current stream
+ *
+ * Return value: the current playback position in the current stream
+ *  in nanoseconds.
  */
-long
+gint64
 rb_player_get_time (RBPlayer *player)
 {
 	RBPlayerIface *iface = RB_PLAYER_GET_IFACE (player);
@@ -426,7 +527,8 @@ rb_player_get_time (RBPlayer *player)
 /**
  * rb_player_multiple_open:
  * @player:	a #RBPlayer
- * @returns:	TRUE if the player supports multiple open streams
+ *
+ * Return value: TRUE if the player supports multiple open streams
  * 		(not necessarily multiple playing streams, though)
  */
 gboolean
@@ -447,24 +549,42 @@ rb_player_multiple_open (RBPlayer *player)
  * @error:	returns error information
  *
  * Creates a new player object.
+ *
+ * Return value: new player object.
  */
 RBPlayer*
 rb_player_new (gboolean want_crossfade, GError **error)
 {
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 	if (want_crossfade)
 		return rb_player_gst_xfade_new (error);
 	else
-#endif
 		return rb_player_gst_new (error);
 }
 
+/**
+ * _rb_player_emit_eos:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ *
+ * Emits the 'eos' signal for a stream.  To be used by
+ * implementations only.
+ */
 void
 _rb_player_emit_eos (RBPlayer *player, gpointer stream_data)
 {
 	g_signal_emit (player, signals[EOS], 0, stream_data);
 }
 
+/**
+ * _rb_player_emit_info:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @field: updated metadata field
+ * @value: metadata field value
+ *
+ * Emits the 'info' signal for a stream.  To be used by
+ * implementations only.
+ */
 void
 _rb_player_emit_info (RBPlayer *player,
 		      gpointer stream_data,
@@ -474,36 +594,111 @@ _rb_player_emit_info (RBPlayer *player,
 	g_signal_emit (player, signals[INFO], 0, stream_data, field, value);
 }
 
+/**
+ * _rb_player_emit_buffering:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @progress: current buffering progress.
+ *
+ * Emits the 'buffering' signal for a stream.
+ * To be used by implementations only.
+ */
 void
 _rb_player_emit_buffering (RBPlayer *player, gpointer stream_data, guint progress)
 {
 	g_signal_emit (player, signals[BUFFERING], 0, stream_data, progress);
 }
 
+/**
+ * _rb_player_emit_error:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @error: playback error
+ *
+ * Emits the 'error' signal for a stream.
+ * To be used by implementations only.
+ */
 void
 _rb_player_emit_error (RBPlayer *player, gpointer stream_data, GError *error)
 {
 	g_signal_emit (player, signals[ERROR], 0, stream_data, error);
 }
 
+/**
+ * _rb_player_emit_tick:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @elapsed: current playback position
+ * @duration: current perception of the duration of the stream (-1 if not applicable)
+ *
+ * Emits the 'tick' signal for a stream.
+ * To be used by implementations only.
+ */
 void
-_rb_player_emit_tick (RBPlayer *player, gpointer stream_data, long elapsed, long duration)
+_rb_player_emit_tick (RBPlayer *player, gpointer stream_data, gint64 elapsed, gint64 duration)
 {
 	g_signal_emit (player, signals[TICK], 0, stream_data, elapsed, duration);
 }
 
+/**
+ * _rb_player_emit_event:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @name: event name
+ * @data: event data
+ *
+ * Emits the 'event' signal for a stream.
+ * To be used by implementations only.
+ */
 void
 _rb_player_emit_event (RBPlayer *player, gpointer stream_data, const char *name, gpointer data)
 {
 	g_signal_emit (player, signals[EVENT], g_quark_from_string (name), stream_data, data);
 }
 
+/**
+ * _rb_player_emit_playing_stream:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the new playing stream
+ *
+ * Emits the 'playing-stream' signal to indicate the current
+ * playing stream has changed.  To be used by implementations only.
+ */
 void
 _rb_player_emit_playing_stream (RBPlayer *player, gpointer stream_data)
 {
 	g_signal_emit (player, signals[PLAYING_STREAM], 0, stream_data);
 }
 
+/**
+ * _rb_player_emit_volume_changed:
+ * @player: a #RBPlayer implementation
+ * @volume: the new volume level
+ *
+ * Emits the 'volume-changed' signal to indicate the output stream
+ * volume has been changed.  To be used by implementations only.
+ */
+void
+_rb_player_emit_volume_changed (RBPlayer *player, float volume)
+{
+	g_signal_emit (player, signals[VOLUME_CHANGED], 0, volume);
+}
+
+/**
+ * _rb_player_emit_image:
+ * @player: a #RBPlayer implementation
+ * @stream_data: data associated with the stream
+ * @image: an image extracted from the stream
+ *
+ * Emits the 'image' signal to notify listeners of an image that
+ * has been extracted from the stream.  To be used by implementations only.
+ */
+void
+_rb_player_emit_image (RBPlayer *player, gpointer stream_data, GdkPixbuf *image)
+{
+	g_signal_emit (player, signals[IMAGE], 0, stream_data, image);
+}
+
 GQuark
 rb_player_error_quark (void)
 {
@@ -536,3 +731,22 @@ rb_player_error_get_type (void)
 	return etype;
 }
 
+GType
+rb_player_play_type_get_type (void)
+{
+	static GType etype = 0;
+
+	if (etype == 0)	{
+		static const GEnumValue values[] = {
+			ENUM_ENTRY (RB_PLAYER_PLAY_REPLACE, "Replace existing stream"),
+			ENUM_ENTRY (RB_PLAYER_PLAY_AFTER_EOS, "Start new stream after EOS of existing stream"),
+			ENUM_ENTRY (RB_PLAYER_PLAY_CROSSFADE, "Crossfade between streams"),
+			{ 0, 0, 0 }
+		};
+
+		etype = g_enum_register_static ("RBPlayerPlayType", values);
+	}
+
+	return etype;
+}
+
