@@ -8,7 +8,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -35,20 +35,15 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#ifdef HAVE_HAL
-#include <libhal.h>
-#include <dbus/dbus.h>
-#endif
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <gpod/itdb.h>
 
 #include "eel-gconf-extensions.h"
 #include "rb-ipod-source.h"
 #include "rb-ipod-db.h"
+#include "rb-ipod-helpers.h"
 #include "rb-debug.h"
 #include "rb-file-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-plugin.h"
 #include "rb-removable-media-manager.h"
 #include "rb-ipod-static-playlist-source.h"
@@ -72,11 +67,7 @@ static void impl_move_to_trash (RBSource *asource);
 static void rb_ipod_load_songs (RBiPodSource *source);
 static void impl_delete_thyself (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
-#ifdef HAVE_HAL
-static gboolean hal_udi_is_ipod (const char *udi);
-#endif
 
-#ifdef ENABLE_IPOD_WRITING
 static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
@@ -98,7 +89,7 @@ static gboolean rb_ipod_song_artwork_add_cb (RhythmDB *db,
                                              const gchar *property_name,
                                              const GValue *metadata,
                                              RBiPodSource *isource);
-#endif
+
 static RhythmDB *get_db_for_source (RBiPodSource *source);
 
 struct _PlayedEntry {
@@ -124,12 +115,10 @@ typedef struct
 	GQueue *offline_plays;
 } RBiPodSourcePrivate;
 
-#ifdef ENABLE_IPOD_WRITING
 typedef struct {
 	RBiPodSourcePrivate *priv;
 	GdkPixbuf *pixbuf;
 } RBiPodSongArtworkAddData;
-#endif
 
 RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
 		      rb_ipod_source,
@@ -157,15 +146,11 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
 
-#ifdef ENABLE_IPOD_WRITING
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
+	rms_class->impl_should_paste = rb_removable_media_source_should_paste_no_duplicate;
 	rms_class->impl_track_added = impl_track_added;
 	rms_class->impl_build_dest_uri = impl_build_dest_uri;
 	rms_class->impl_get_mime_types = impl_get_mime_types;
-#else
-	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
-	rms_class->impl_track_added = NULL;
-#endif
 
 	browser_source_class->impl_get_paned_key = impl_get_paned_key;
 
@@ -178,6 +163,11 @@ rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
+	if (priv->ipod_db == NULL) {
+		rb_debug ("can't change ipod name with no ipod db");
+		return;
+	}
+
 	rb_ipod_db_set_ipod_name (priv->ipod_db, name);
 }
 
@@ -214,14 +204,12 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 
 	rb_ipod_load_songs (source);
 
-#ifdef ENABLE_IPOD_WRITING                                      
         RhythmDB *db = get_db_for_source (RB_IPOD_SOURCE (source));
         g_signal_connect_object (db,
                                  "entry-extra-metadata-notify::rb:coverArt",
                                  G_CALLBACK (rb_ipod_song_artwork_add_cb),
                                  RB_IPOD_SOURCE(source), 0);
         g_object_unref (G_OBJECT (db));
-#endif
 
 	return G_OBJECT (source);
 }
@@ -269,19 +257,22 @@ rb_ipod_source_dispose (GObject *object)
 }
 
 RBRemovableMediaSource *
-rb_ipod_source_new (RBShell *shell,
-		    GnomeVFSVolume *volume)
+rb_ipod_source_new (RBPlugin *plugin,
+		    RBShell *shell,
+		    GMount *mount)
 {
 	RBiPodSource *source;
 	RhythmDBEntryType entry_type;
 	RhythmDB *db;
+	GVolume *volume;
 	char *name;
 	char *path;
 
-	g_assert (rb_ipod_is_volume_ipod (volume));
-
+	volume = g_mount_get_volume (mount);
+	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	g_object_unref (volume);
+	
 	g_object_get (shell, "db", &db, NULL);
-	path = gnome_vfs_volume_get_device_path (volume);
 	name = g_strdup_printf ("ipod: %s", path);
 	entry_type =  rhythmdb_entry_register_type (db, name);
 	entry_type->save_to_disk = FALSE;
@@ -291,8 +282,9 @@ rb_ipod_source_new (RBShell *shell,
 	g_free (path);
 
 	source = RB_IPOD_SOURCE (g_object_new (RB_TYPE_IPOD_SOURCE,
+				               "plugin", plugin,
 					       "entry-type", entry_type,
-					       "volume", volume,
+					       "mount", mount,
 					       "shell", shell,
 					       "source-group", RB_SOURCE_GROUP_DEVICES,
 					       NULL));
@@ -455,8 +447,6 @@ load_ipod_playlists (RBiPodSource *source)
 
 }
 
-#ifdef ENABLE_IPOD_WRITING
-
 /* FIXME:  these should go away once we compile against new-enough libgpod */
 #define MEDIATYPE_AUDIO         0x0001
 #define MEDIATYPE_PODCAST       0x0004
@@ -473,6 +463,8 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->artist = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST);
 	track->sort_artist = rhythmdb_entry_dup_string (entry,
 	                                                RHYTHMDB_PROP_ARTIST_SORTNAME);
+	track->sort_album = rhythmdb_entry_dup_string (entry,
+	                                                RHYTHMDB_PROP_ALBUM_SORTNAME);
 	track->genre = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_GENRE);
 	track->filetype = g_strdup (mimetype);
 	track->size = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
@@ -482,9 +474,8 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->track_nr = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER);
 	track->bitrate = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE);
 	track->year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
-	track->time_added = itdb_time_get_mac_time ();
+	track->time_added = time (NULL);
 	track->time_played = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_LAST_PLAYED);
-	track->time_played = itdb_time_host_to_mac (track->time_played);
 	track->rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING);
 	track->rating *= ITDB_RATING_STEP;
 	track->app_rating = track->rating;
@@ -493,14 +484,12 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	if (rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
 		track->mediatype = MEDIATYPE_PODCAST;
 		track->time_released = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
-		track->time_released = itdb_time_host_to_mac (track->time_released);
 	} else {
 		track->mediatype = MEDIATYPE_AUDIO;
 	}
 
 	return track;
 }
-#endif
 
 static void add_offline_played_entry (RBiPodSource *source,
 				      RhythmDBEntry *entry,
@@ -654,7 +643,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	if (song->time_added != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
-		g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_added));
+		g_value_set_ulong (&value, song->time_added);
 		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_FIRST_SEEN,
 					       &value);
@@ -665,7 +654,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	if (song->time_played != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
-		g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_played));
+		g_value_set_ulong (&value, song->time_played);
 		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_LAST_PLAYED,
 					       &value);
@@ -686,6 +675,12 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
                                        song->sort_artist);
         }
 
+        if (song->sort_album != NULL) {
+                entry_set_string_prop (RHYTHMDB (db), entry,
+                                       RHYTHMDB_PROP_ALBUM_SORTNAME,
+                                       song->sort_album);
+        }
+
 	entry_set_string_prop (RHYTHMDB (db), entry,
 			       RHYTHMDB_PROP_ALBUM, song->album);
 
@@ -881,7 +876,7 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 			if (old_lastplay != new_lastplay) {
 				track = g_hash_table_lookup (priv->entry_map, 
 							     entry);
-				track->time_played = itdb_time_host_to_mac (new_lastplay);
+				track->time_played = new_lastplay;
 				rb_debug ("last play time changed, saving db");
 				rb_ipod_db_save_async (priv->ipod_db);
 			} else {
@@ -932,10 +927,10 @@ static void
 rb_ipod_load_songs (RBiPodSource *source)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
-	GnomeVFSVolume *volume;
+	GMount *mount;
 
-	g_object_get (source, "volume", &volume, NULL);
- 	priv->ipod_db = rb_ipod_db_new (volume);
+	g_object_get (source, "mount", &mount, NULL);
+ 	priv->ipod_db = rb_ipod_db_new (mount);
 	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
@@ -951,148 +946,9 @@ rb_ipod_load_songs (RBiPodSource *source)
                                   NULL);
 		priv->load_idle_id = g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
 	}
-	g_object_unref (volume);
+	g_object_unref (mount);
 }
 
-static gchar *
-rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
-{
-	gchar *mount_point_uri;
-	gchar *mount_point;
-	gchar *result;
-
-	mount_point_uri = gnome_vfs_volume_get_activation_uri (volume);
-	if (mount_point_uri == NULL) {
-		return NULL;
-	}
-	mount_point = g_filename_from_uri (mount_point_uri, NULL, NULL);
-	g_free (mount_point_uri);
-	if (mount_point == NULL) {
-		return NULL;
-	}
-
-	result = itdb_get_itunes_dir(mount_point);
-
-	g_free (mount_point);
-	return result;
-}
-
-static gboolean
-rb_ipod_volume_has_ipod_db (GnomeVFSVolume *volume)
-{
-	char *itunesdb_path;
-	gboolean result;
-
-	itunesdb_path = rb_ipod_get_itunesdb_path (volume);
-
-	if (itunesdb_path != NULL) {
-		result = g_file_test (itunesdb_path, G_FILE_TEST_EXISTS);
-	} else {
-		result = FALSE;
-	}
-	g_free (itunesdb_path);
-
-	return result;
-}
-
-gboolean
-rb_ipod_is_volume_ipod (GnomeVFSVolume *volume)
-{
-#ifdef HAVE_HAL
-	gchar *udi;
-#endif
-	if (gnome_vfs_volume_get_volume_type (volume) != GNOME_VFS_VOLUME_TYPE_MOUNTPOINT) {
-		return FALSE;
-	}
-
-#ifdef HAVE_HAL
-	udi = gnome_vfs_volume_get_hal_udi (volume);
-	if (udi != NULL) {
-		gboolean result;
-
-		result = hal_udi_is_ipod (udi);
-		g_free (udi);
-		if (result == FALSE) {
-			return FALSE;
-		}
-	}
-#endif
-
-	return rb_ipod_volume_has_ipod_db (volume);
-}
-
-#ifdef HAVE_HAL
-
-static gboolean
-hal_udi_is_ipod (const char *udi)
-{
-	LibHalContext *ctx;
-	DBusConnection *conn;
-	char *parent_udi;
-	char *parent_name;
-	gboolean result;
-	DBusError error;
-	gboolean inited = FALSE;
-
-	result = FALSE;
-	dbus_error_init (&error);
-
-	conn = NULL;
-	parent_udi = NULL;
-	parent_name = NULL;
-
-	ctx = libhal_ctx_new ();
-	if (ctx == NULL) {
-		/* FIXME: should we return an error somehow so that we can
-		 * fall back to a check for iTunesDB presence instead ?
-		 */
-		rb_debug ("cannot connect to HAL");
-		goto end;
-	}
-	conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (conn == NULL || dbus_error_is_set (&error))
-		goto end;
-
-	libhal_ctx_set_dbus_connection (ctx, conn);
-	if (!libhal_ctx_init (ctx, &error) || dbus_error_is_set (&error))
-		goto end;
-
-	inited = TRUE;
-	parent_udi = libhal_device_get_property_string (ctx, udi,
-			"info.parent", &error);
-	if (parent_udi == NULL || dbus_error_is_set (&error))
-		goto end;
-
-	parent_name = libhal_device_get_property_string (ctx, parent_udi,
-			"storage.model", &error);
-	if (parent_name == NULL || dbus_error_is_set (&error))
-		goto end;
-
-	if (strcmp (parent_name, "iPod") == 0)
-		result = TRUE;
-
-end:
-	g_free (parent_udi);
-	g_free (parent_name);
-
-	if (dbus_error_is_set (&error)) {
-		rb_debug ("Error: %s\n", error.message);
-		dbus_error_free (&error);
-		dbus_error_init (&error);
-	}
-
-	if (ctx) {
-		if (inited)
-			libhal_ctx_shutdown (ctx, &error);
-		libhal_ctx_free(ctx);
-	}
-
-	dbus_error_free (&error);
-
-	return result;
-}
-#endif
-
 static GList*
 impl_get_ui_actions (RBSource *source)
 {
@@ -1122,28 +978,22 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-static void
-impl_move_to_trash (RBSource *asource)
+void
+rb_ipod_source_trash_entries (RBiPodSource *source, GList *entries)
 {
-	GList *sel, *tem;
-	RBEntryView *songs;
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDB *db;
-	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (asource);
-	RBiPodSource *source = RB_IPOD_SOURCE (asource);
+	GList *tem;
 
 	db = get_db_for_source (source);
-
-	songs = rb_source_get_entry_view (RB_SOURCE (asource));
-	sel = rb_entry_view_get_selected_entries (songs);
-	for (tem = sel; tem != NULL; tem = tem->next) {
+	for (tem = entries; tem != NULL; tem = tem->next) {
 		RhythmDBEntry *entry;
 		const gchar *uri;
 		gchar *file;
 		Itdb_Track *track;
 
 		entry = (RhythmDBEntry *)tem->data;
-		uri = rhythmdb_entry_get_string (entry,
-						 RHYTHMDB_PROP_LOCATION);
+		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 		track = g_hash_table_lookup (priv->entry_map, entry);
 		if (track == NULL) {
 			g_warning ("Couldn't find track on ipod! (%s)", uri);
@@ -1157,15 +1007,26 @@ impl_move_to_trash (RBSource *asource)
 			g_unlink (file);
 		g_free (file);
 		rhythmdb_entry_delete (db, entry);
-		rhythmdb_commit (db);
 	}
 
+	rhythmdb_commit (db);
 	g_object_unref (db);
+}
 
+static void
+impl_move_to_trash (RBSource *source)
+{
+	GList *sel;
+	RBEntryView *songs;
+
+	songs = rb_source_get_entry_view (source);
+	sel = rb_entry_view_get_selected_entries (songs);
+	rb_ipod_source_trash_entries (RB_IPOD_SOURCE (source), sel);
+
+	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
 	g_list_free (sel);
 }
 
-#ifdef ENABLE_IPOD_WRITING
 static char *
 impl_build_dest_uri (RBRemovableMediaSource *source,
 		     RhythmDBEntry *entry,
@@ -1177,6 +1038,10 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 	char *dest;
 	const char *mount_path;
 
+	if (priv->ipod_db == NULL) {
+		return NULL;
+	}
+
 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
 	dest = ipod_get_filename_for_uri (mount_path,  uri, 
@@ -1478,15 +1343,22 @@ get_ipod_filename (const char *mount_point, const char *filename)
 	g_free (dirname);
 
 	if (strlen (result) >= IPOD_MAX_PATH_LEN) {
-		char *ext;
+		char *ext, *suffix;
 
 		ext = strrchr (result, '.');
 		if (ext == NULL) {
+			suffix = result + IPOD_MAX_PATH_LEN - 4;
 			result [IPOD_MAX_PATH_LEN - 1] = '\0';
 		} else {
+			suffix = result + IPOD_MAX_PATH_LEN - 4 - strlen(ext);
 			memmove (&result[IPOD_MAX_PATH_LEN - strlen (ext) - 1] ,
 				 ext, strlen (ext) + 1);
 		}
+
+		/* Add suffix to reduce the chance of a name collision with truncated name */
+		suffix[0] = '~';
+		suffix[1] = 'A' + g_random_int_range (0, 26);
+		suffix[2] = 'A' + g_random_int_range (0, 26);
 	}
 
 	tmp = g_build_filename (mount_point, result, NULL);
@@ -1571,7 +1443,7 @@ ipod_get_filename_for_uri (const gchar *mount_point,
 	if (escaped == NULL) {
 		return NULL;
 	}
-	filename = gnome_vfs_unescape_string (escaped, G_DIR_SEPARATOR_S);
+	filename = g_uri_unescape_string (escaped, NULL);
 	g_free (escaped);
 	if (filename == NULL) {
 		return NULL;
@@ -1624,7 +1496,6 @@ ipod_path_from_unix_path (const gchar *mount_point, const gchar *unix_path)
 
 	return ipod_path;
 }
-#endif
 
 static void
 impl_delete_thyself (RBSource *source)
@@ -1665,8 +1536,6 @@ impl_delete_thyself (RBSource *source)
 	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
 }
 
-#ifdef ENABLE_IPOD_WRITING
-
 static GList *
 impl_get_mime_types (RBRemovableMediaSource *source)
 {
@@ -1679,8 +1548,6 @@ impl_get_mime_types (RBRemovableMediaSource *source)
 	return ret;
 }
 
-#endif
-
 
 void
 rb_ipod_source_new_playlist (RBiPodSource *source)
@@ -1688,6 +1555,11 @@ rb_ipod_source_new_playlist (RBiPodSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	Itdb_Playlist *ipod_playlist;
 
+	if (priv->ipod_db == NULL) {
+		rb_debug ("can't create new ipod playlist with no ipod db");
+		return;
+	}
+
 	ipod_playlist = itdb_playlist_new (_("New playlist"), FALSE);
 	rb_ipod_db_add_playlist (priv->ipod_db, ipod_playlist);
 	add_rb_playlist (source, ipod_playlist);
@@ -1704,3 +1576,127 @@ rb_ipod_source_remove_playlist (RBiPodSource *ipod_source,
 	rb_ipod_db_remove_playlist (priv->ipod_db, rb_ipod_static_playlist_source_get_itdb_playlist (playlist_source));
 }
 
+static void
+rb_ipod_info_response_cb (GtkDialog *dialog,
+ 			  int response_id,
+ 			  RBiPodSource *source)
+{
+	if (response_id == GTK_RESPONSE_CLOSE) {
+ 		
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+	}
+}
+
+static gboolean
+ipod_name_changed_cb (GtkWidget     *widget,
+ 		      GdkEventFocus *event,
+ 		      gpointer       user_data) 
+{
+	g_object_set (RB_SOURCE (user_data), "name", 
+		      gtk_entry_get_text (GTK_ENTRY (widget)),
+		      NULL);
+	return FALSE;
+}
+
+void
+rb_ipod_source_show_properties (RBiPodSource *source)
+{
+	GtkBuilder *builder;
+	GObject *dialog;
+	GObject *label;
+	char *text;
+	const gchar *mp;
+	char *used;
+	char *capacity;
+	char *builder_file;
+ 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	Itdb_Device *ipod_dev;
+	RBPlugin *plugin;
+
+	if (priv->ipod_db == NULL) {
+		rb_debug ("can't show ipod properties with no ipod db");
+		return;
+	}
+
+	ipod_dev = rb_ipod_db_get_device (priv->ipod_db);
+
+	g_object_get (source, "plugin", &plugin, NULL);
+	builder_file = rb_plugin_find_file (plugin, "ipod-info.ui");
+	g_object_unref (plugin);
+
+	if (builder_file == NULL) {
+		g_warning ("Couldn't find ipod-info.ui");
+		return;
+	}
+
+	builder = rb_builder_load (builder_file, NULL);
+	g_free (builder_file);
+
+ 	if (builder == NULL) {
+ 		rb_debug ("Couldn't load ipod-info.ui");
+ 		return;
+ 	}
+	
+ 	dialog = gtk_builder_get_object (builder, "ipod-information");
+ 	g_signal_connect_object (dialog,
+ 				 "response",
+ 				 G_CALLBACK (rb_ipod_info_response_cb),
+ 				 source, 0);
+ 
+ 	label = gtk_builder_get_object (builder, "label-number-track-number");
+ 	text = g_strdup_printf ("%u", g_list_length (rb_ipod_db_get_tracks(priv->ipod_db) ));
+ 	gtk_label_set_text (GTK_LABEL (label), text);
+ 	g_free (text);
+ 
+ 	label = gtk_builder_get_object (builder, "entry-ipod-name");
+ 	gtk_entry_set_text (GTK_ENTRY (label), rb_ipod_db_get_ipod_name (priv->ipod_db));
+ 	g_signal_connect (label, "focus-out-event",
+ 			  (GCallback)ipod_name_changed_cb, source);
+ 
+ 	label = gtk_builder_get_object (builder, "label-number-playlist-number");
+ 	text = g_strdup_printf ("%u", g_list_length (rb_ipod_db_get_playlists (priv->ipod_db)));
+ 	gtk_label_set_text (GTK_LABEL (label), text);
+ 	g_free (text);
+ 
+ 	label = gtk_builder_get_object (builder, "label-mount-point-value");
+	mp = rb_ipod_db_get_mount_path (priv->ipod_db);
+ 	gtk_label_set_text (GTK_LABEL (label), mp);
+
+	label = gtk_builder_get_object (builder, "progressbar-ipod-usage");
+	used = g_format_size_for_display (rb_ipod_helpers_get_capacity (mp) - rb_ipod_helpers_get_free_space (mp));
+	capacity = g_format_size_for_display (rb_ipod_helpers_get_capacity(mp));
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (label), 
+				       (double)(rb_ipod_helpers_get_capacity (mp) - rb_ipod_helpers_get_free_space (mp))/(double)rb_ipod_helpers_get_capacity (mp));
+	/* Translators: this is used to display the amount of storage space
+	 * used and the total storage space on an iPod.
+	 */
+	text = g_strdup_printf (_("%s of %s"), used, capacity);
+	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (label), text);
+	g_free (text);
+	g_free (capacity);
+	g_free (used);
+
+	label = gtk_builder_get_object (builder, "label-device-node-value");
+	text = rb_ipod_helpers_get_device (RB_SOURCE(source));
+	gtk_label_set_text (GTK_LABEL (label), text);
+	g_free (text);
+
+ 	label = gtk_builder_get_object (builder, "label-ipod-model-value");
+ 	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "ModelNumStr"));
+
+ 	label = gtk_builder_get_object (builder, "label-database-version-value");
+	text = g_strdup_printf ("%u", rb_ipod_db_get_database_version (priv->ipod_db));
+ 	gtk_label_set_text (GTK_LABEL (label), text);
+	g_free (text);
+
+ 	label = gtk_builder_get_object (builder, "label-serial-number-value");
+	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "pszSerialNumber"));
+
+ 	label = gtk_builder_get_object (builder, "label-firmware-version-value");
+	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "VisibleBuildID"));
+
+ 	gtk_widget_show (GTK_WIDGET (dialog));
+
+	g_object_unref (builder);
+}
+
