@@ -30,7 +30,7 @@
 # - check that the lyrics returned even remotely match the request?
 
 import os, re
-import gtk, gtk.glade
+import gtk
 import gconf
 import rhythmdb, rb
 
@@ -74,6 +74,16 @@ def create_lyrics_view():
 	return (vbox, tview.get_buffer(), tview)
 
 def parse_song_data(artist, title):
+
+	# don't search for 'unknown' when we don't have the artist or title information
+	if artist == _("Unknown"):
+		artist = ""
+	if title == _("Unknown"):
+		title = ""
+
+	# convert to lowercase
+	artist = artist.lower()
+	title = title.lower()
 	
 	# replace ampersands and the like
 	for exp in LYRIC_ARTIST_REPLACE:
@@ -93,7 +103,7 @@ def parse_song_data(artist, title):
 	
 def build_cache_path(artist, title):
 	folder = gconf.client_get_default().get_string(gconf_keys['folder'])
-	if folder is None:
+	if folder is None or folder == "":
 		folder = os.path.join(rb.user_cache_dir(), "lyrics")
 
 	lyrics_folder = os.path.expanduser (folder)
@@ -111,8 +121,8 @@ class LyricGrabber(object):
 		self.db = db
 		self.entry = entry
 		
-		self.artist = self.db.entry_get(self.entry, rhythmdb.PROP_ARTIST).lower()
-		self.title = self.db.entry_get(self.entry, rhythmdb.PROP_TITLE).lower()
+		self.artist = self.db.entry_get(self.entry, rhythmdb.PROP_ARTIST)
+		self.title = self.db.entry_get(self.entry, rhythmdb.PROP_TITLE)
 
 		(self.artist, self.title) = parse_song_data(self.artist, self.title)
 
@@ -129,21 +139,22 @@ class LyricGrabber(object):
 		if status:
 			l = rb.Loader()
 			l.get_url(self.cache_path, callback)
+		elif cache_only:
+			self.callback(_("No lyrics found"))
+		elif self.artist == "" and self.title == "":
+			self.callback(_("No lyrics found"))
 		else:
-			if cache_only:
-				self.callback(_("No lyrics found"))
-			else:
-				def lyric_callback (text):
-					if text is not None:
-						f = file (self.cache_path, 'w')
-						f.write (text)
-						f.close ()
-						self.callback(text)
-					else:
-						self.callback(_("No lyrics found"))
+			def lyric_callback (text):
+				if text is not None:
+					f = file (self.cache_path, 'w')
+					f.write (text)
+					f.close ()
+					self.callback(text)
+				else:
+					self.callback(_("No lyrics found"))
 
-				parser = LyricsParse.Parser(gconf_keys, self.artist, self.title)
-				parser.get_lyrics(lyric_callback)
+			parser = LyricsParse.Parser(gconf_keys, self.artist, self.title)
+			parser.get_lyrics(lyric_callback)
 
 class LyricPane(object):
 	def __init__(self, db, song_info):
@@ -224,8 +235,8 @@ class LyricPane(object):
 
 	def build_path(self):
 
-		artist = self.db.entry_get(self.entry, rhythmdb.PROP_ARTIST).lower()
-		title = self.db.entry_get(self.entry, rhythmdb.PROP_TITLE).lower()
+		artist = self.db.entry_get(self.entry, rhythmdb.PROP_ARTIST)
+		title = self.db.entry_get(self.entry, rhythmdb.PROP_TITLE)
 		(artist, title) = parse_song_data(artist, title)
 		cache_path = build_cache_path(artist, title)
 		self.cache_path = cache_path
@@ -248,13 +259,16 @@ class LyricPane(object):
 		self.visible = 1
 		self.get_lyrics()
 
+	def __got_lyrics(self, text):
+		self.buffer.set_text(text)
+
 	def get_lyrics(self):
 		if self.entry is None:
 			return
 
 		self.buffer.set_text(_("Searching for lyrics..."));
 		lyrics_grabber = LyricGrabber(self.db, self.entry)
-		lyrics_grabber.search_lyrics(self.buffer.set_text)
+		lyrics_grabber.search_lyrics(self.__got_lyrics)
 			
 
 class LyricWindow (gtk.Window):
@@ -327,8 +341,8 @@ class LyricsDisplayPlugin(rb.Plugin):
 
 	def create_configure_dialog(self, dialog=None):
 		if not dialog:
-			glade_file = self.find_file("lyrics-prefs.glade")
-			dialog = LyricsConfigureDialog (glade_file, gconf_keys).get_dialog()
+			builder_file = self.find_file("lyrics-prefs.ui")
+			dialog = LyricsConfigureDialog (builder_file, gconf_keys).get_dialog()
 		dialog.present()
 		return dialog
 	
@@ -339,6 +353,11 @@ class LyricsDisplayPlugin(rb.Plugin):
 		else:
 			self.action.set_sensitive (False)
 
+	def __got_lyrics(self, text):
+		# don't do anything if the window has already been destroyed
+		if self.window is not None:
+			self.window.buffer.set_text (text)
+
 	def update_song_lyrics(self, entry):
 		if entry == self.current_entry:
 			return
@@ -353,7 +372,7 @@ class LyricsDisplayPlugin(rb.Plugin):
 
 		self.window.s_title(title, artist)
 		lyrics_grabber = LyricGrabber(db, entry)
-		lyrics_grabber.search_lyrics(self.window.buffer.set_text)
+		lyrics_grabber.search_lyrics(self.__got_lyrics)
 
 	def show_song_lyrics (self, action, shell):
 
@@ -375,7 +394,7 @@ class LyricsDisplayPlugin(rb.Plugin):
 		self.window.s_title(title, artist)
 		self.window.connect("destroy", self.window_deleted)
 		lyrics_grabber = LyricGrabber(db, entry)
-		lyrics_grabber.search_lyrics(self.window.buffer.set_text)
+		lyrics_grabber.search_lyrics(self.__got_lyrics)
 
 	def window_deleted (self, window):
 		print "lyrics window destroyed"
